/*
  Trata matriz Jubete
  x = columna
  y = fila
  n = dimensiones de la columna
  m = dimensiones de la fila
*/

#include "jubete.h"

// ----------------------------------
// n, m: las dimensiones de la matriz
PMatrizJubete crearMatriz(int n, int m) {
     PMatrizJubete A = 0;
     if(n > 0 && m > 0) {
          A = malloc(sizeof(TMatrizJubete));
          A->n = n; A->m = m;
          A->M = malloc(sizeof(double)*n*m);
     }
     return A;
}

PMatrizJubete crearMatrizIdentidad(int n, int m) {
     PMatrizJubete A = crearMatriz(n, m);
     int x, y;
     for(x=0; x < n; x++) {
         for(y=0; y < m; y++) {
             if(x == y) setM(A, x, y, 1);
             else setM(A, x, y, 0);
         }   	
     }
     return A;
}

double getM(PMatrizJubete A, int x, int y) {
       double r = 0;
       if(A) {
             r = A->M[A->m*y + x];
       }
       return r;
}

void setM(PMatrizJubete A, int x, int y, double value) {
       if(A) {
             A->M[A->m*y + x] = value;
       }
}

double * getMY(PMatrizJubete A, int y) {
     double *v = 0;
       if(A) {
             v = &A->M[A->m*y];
       }
     return v;
}

// ----------------------------------
void poblarMatriz(PMatrizJubete A) {
     int x, y;
     if(A) {
            for(x = 0; x < A->n; x++) 
                for(y = 0; y < A->m; y++) 
                    setM(A, x, y, (rand() % 100)/10.0);
     }
}
     
// ----------------------------------
void muestraMatriz(PMatrizJubete A) {
     int x, y;
     if(A) {
            for(y = 0; y < A->m; y++) {
                for(x = 0; x < A->n; x++) {
                    printf("%0.2f\t", getM(A, x, y));
                }
                printf("\n");
            }
     }
}

// ----------------------------------
void destruirMatriz(PMatrizJubete A) {
     if(A) {
           if(A->M) free(A->M);
           free(A);
     }
}

//--------------------------------------------------
// Esta funcion halla la inversa de una matriz a travez del algoritmo de Jubete
// A: la matriz
//--------------------------------------------------
PMatrizJubete invJubete(PMatrizJubete A) {
   PMatrizJubete B, C;
   double *TIJ, *filaTIJ;
   double pivote, tij=0, nuevo;
   int x, y, i;
   int contpivote=0, colpivote;
   
   if(A->m == A->n) {
   
       // se crea la matriz identidad para la iteracion 0
       B = crearMatriz(A->n, 1);
       C = crearMatrizIdentidad(A->n, A->m);
       
       TIJ = getMY(B, 0); // primeira fila da matriz B
       
	   for(i = 0; i < A->n; i++) {
             
           // pobla columnas tij
           
           filaTIJ = getMY(A, i);
           
	       for(x=0; x < A->n; x++, tij=0)  {
   	           for(y=0; y < A->m; y++ ) {
   	                    tij += filaTIJ[y] * getM(C, x, y);
               }
      	       TIJ[x] = tij;
	       }
	       
	       // busca primeira columna no zero (pivote)
	       
	       pivote=0;
	       for(x=contpivote; x < A->n; x++) {
               if(TIJ[x] != 0) {
                   pivote = TIJ[x];
             	   colpivote = x;
                   break;
               } 
           }

           // conclui calculo da inversa
           
   	       if(pivote == 0) {           // se no encontrou columna pivote...
               if(i == A->n) {
                  destruirMatriz(B);
                  destruirMatriz(C);
	 	          return 0;            // no invertible
		       } 
           } else {                    // ms, se existe pivote...
               contpivote++;
	           for(y=0; y < A->n; y++) {
                  setM(C, colpivote, y, getM(C, colpivote, y) / pivote);
               }
               for(x=0; x < A->n; x++) {
                  for(y=0; y < A->n; y++) {
               		  if(x != colpivote) {
                         nuevo = getM(C, x, y) - (TIJ[x] * getM(C, colpivote, y));
                         setM(C, x, y, nuevo);
                      }
                  }
               }
           }
           
       }
       
       destruirMatriz(B);
       
       return C;       
   } 
   
   return 0;      // si la matriz no es cuadrada 
}

